#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Float_Input.H>
#include <FL/Fl_Slider.H>
#include <stdio.h>

class MyInput : public Fl_Float_Input {
public:
    MyInput(int x, int y, int w, int h, const char *label = 0);
    int handle( int event );
};

MyInput::MyInput(int x, int y, int w, int h, const char *label):Fl_Float_Input(x, y, w, h, label)
{
}

int MyInput::handle(int event)
{
    if ( event == FL_UNFOCUS )
        position( 0 );

    return Fl_Float_Input::handle( event );
}

class SliderInput : public Fl_Group {
    MyInput *input;
    Fl_Slider *slider;

    void Slider_CB2() {
        static int recurse = 0;
        if ( recurse ) {
            return;
        } else {
            recurse = 1;
            char s[80];
            snprintf(s, sizeof(s), "%.3f", slider->value());
            input->value(s);
            input->position(0);
            recurse = 0;
        }
    }

    static void Slider_CB(Fl_Widget *w, void *data) {
        ((SliderInput*)data)->Slider_CB2();
    }

    void Input_CB2() {
        static int recurse = 0;
        if ( recurse ) {
            return;
        } else {
            recurse = 1;
            double val = 0;
            if ( sscanf(input->value(), "%f", &val) != 1 ) {
                val = 0;
            }
            slider->value(val);
            recurse = 0;
        }
    }

    static void Input_CB(Fl_Widget *w, void *data) {
        ((SliderInput*)data)->Input_CB2();
    }

public:
    SliderInput(int x, int y, int w, int h, const char *l=0) : Fl_Group(x,y,w,h,l) {
        int in_w = 40;
        input  = new MyInput(x, y, in_w, h);
        input->callback(Input_CB, (void*)this);
        input->when(FL_WHEN_ENTER_KEY|FL_WHEN_RELEASE);

        slider = new Fl_Slider(x+in_w, y, w-in_w, h);
        slider->type(1);
        slider->callback(Slider_CB, (void*)this);

        bounds(1, 10);
        value(5);
        end();
    }

    // MINIMAL ACCESSORS --  Add your own as needed
    int  value() const    { return((int)(slider->value() + 0.5)); }
    void value(int val)   { slider->value(val); Slider_CB2(); }
    void minumum(int val) { slider->minimum(val); }
    int  minumum() const  { return((int)slider->minimum()); }
    void maximum(int val) { slider->maximum(val); }
    int  maximum() const  { return((int)slider->maximum()); }
    void bounds(int low, int high) { slider->bounds(low, high); }
};

int main() {
    Fl_Window win(240,200);
    SliderInput *si = new SliderInput(20,20,200,50,"Slider Input");
    si->bounds(0,10000);
    si->value(50);

    SliderInput *si2 = new SliderInput(20,100,200,50,"Slider2 Input");
    si2->bounds(0,10000);
    si2->value(150);

    win.show();
    return(Fl::run());
}
 